# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.1.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._types import *
else:
    from _types import *


import enum
AccessMode = enum.IntEnum('AccessMode', _enum_list_AccessMode())
AccessMode.__doc__ = "An identifier for each mode of metadata support."


import enum
ByteOrder = enum.IntEnum('ByteOrder', _enum_list_ByteOrder())
ByteOrder.__doc__ = "Type to express the byte order (little or big endian)."


import enum
MetadataId = enum.IntEnum('MetadataId', _enum_list_MetadataId())
MetadataId.__doc__ = "An identifier for each type of metadata."


import enum
TypeId = enum.IntEnum('TypeId', _enum_list_TypeId())
TypeId.__doc__ = "Exiv2 value type identifiers.\n"
"\nUsed primarily as identifiers when creating Exiv2 Value instances. See"
"\nexiv2.Value.create(). 0x0000 to 0xffff are reserved for TIFF (Exif) types."


