# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.1.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
import exiv2.types

# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._value import *
else:
    from _value import *


import enum
CharsetId = enum.IntEnum('CharsetId', _enum_list_CharsetId())
CharsetId.__doc__ = "Character set identifiers for the character sets defined by Exif."


import enum
XmpArrayType = enum.IntEnum('XmpArrayType', _enum_list_XmpArrayType())
XmpArrayType.__doc__ = "XMP array types."


import enum
XmpStruct = enum.IntEnum('XmpStruct', _enum_list_XmpStruct())
XmpStruct.__doc__ = "XMP structure indicator."


